==> tutti-ui-swing-2.8-sources.jar: fr/ifremer/tutti/ui/swing/spatial/DmdCoordinate.java <==
package fr.ifremer.tutti.ui.swing.spatial;

/*
 * #%L
 * Tutti :: UI
 * $Id: DmdCoordinate.java 1316 2013-10-25 16:43:35Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-2.8/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/spatial/DmdCoordinate.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import org.jdesktop.beans.AbstractSerializableBean;

/**
 * Geo coordinate in degree decimal, minute format.
 * <p/>
 * Created on 10/23/13.
 *
 * @author Tony Chemit <chemit@codelutin.com>
 * @since 2.8
 */
public class DmdCoordinate extends AbstractSerializableBean {

    private static final long serialVersionUID = 1L;


    public static final String PROPERTY_SIGN = "sign";

    public static final String PROPERTY_DEGREE = "degree";

    public static final String PROPERTY_MINUTE = "minute";

    public static final String PROPERTY_DECIMAL = "decimal";

    protected boolean sign;

    protected Integer degree;

    protected Integer minute;

    protected Integer decimal;

    public static DmdCoordinate empty() {
        DmdCoordinate r = new DmdCoordinate();
        return r;
    }

    /**
     * Methode statique de fabrique de position a partir d'une valeur du format
     * decimal.
     * <p/>
     * Note : Si la valeur (au format decimal) vaut <code>null</code>, alors on
     * reinitialise les composants de la position a <code>null</code> et la
     * methode {@link #isNull()} vaudra alors {@code true}.
     *
     * @param decimal la valeur au format decimal
     * @return une nouvelle instance de position convertie
     */
    public static DmdCoordinate valueOf(Float decimal) {
        DmdCoordinate r = new DmdCoordinate();
        r.fromDecimal(decimal);
        return r;
    }

    /**
     * Methode statique de fabrique de position a partir d'une valeur du format
     * degre décimale minute.
     *
     * @param d  la valeur des degres
     * @param m  la valeur des minutes
     * @param dc la valeur des décimales de minutes
     * @return une nouvelle instance de position convertie
     */
    public static DmdCoordinate valueOf(boolean sign, int d, int m, int dc) {
        DmdCoordinate r = new DmdCoordinate();
        r.setSign(sign);
        r.setDegree(d);
        r.setMinute(m);
        r.setDecimal(dc);
        return r;
    }

    public boolean isSign() {
        return sign;
    }

    public Integer getDegree() {
        return degree;
    }

    public Integer getMinute() {
        return minute;
    }

    public Integer getDecimal() {
        return decimal;
    }

    public void setSign(boolean sign) {
        Object oldValue = isSign();
        this.sign = sign;
        firePropertyChange(PROPERTY_SIGN, oldValue, sign);
    }

    public void setDegree(Integer degree) {
        Object oldValue = getDegree();
        this.degree = degree;
        firePropertyChange(PROPERTY_DEGREE, oldValue, degree);
    }

    public void setMinute(Integer minute) {
        Object oldValue = getMinute();
        this.minute = minute;
        firePropertyChange(PROPERTY_MINUTE, oldValue, minute);
    }

    public void setDecimal(Integer decimal) {
        Object oldValue = getDecimal();
        this.decimal = decimal;
        firePropertyChange(PROPERTY_DECIMAL, oldValue, decimal);
    }

    /**
     * @return {@code true} si aucune composante n'est renseignée,
     * {@code false} autrement.
     */
    public boolean isNull() {
        return degree == null && minute == null && decimal == null;
    }

    public boolean isDegreeNull() {
        return degree == null || degree == 0;
    }

    public boolean isMinuteNull() {
        return minute == null || minute == 0;
    }

    public boolean isDecimalNull() {
        return decimal == null || decimal == 0;
    }

    /**
     * Mets a jour les composants de la position a partir d'une valeur decimal.
     * <p/>
     * Note : Si la valeur (au format decimal) vaut <code>null</code>, alors on
     * reinitialise les composants de la position a <code>null</code> et la
     * methode {@link #isNull()} vaudra alors {@code true}.
     *
     * @param decimalValue la valeur decimale a convertir (qui peut etre nulle).
     */
    public void fromDecimal(Float decimalValue) {
        Integer d = null;
        Integer m = null;
        Integer dc = null;
        boolean si = false;
        if (decimalValue != null) {
            si = decimalValue < 0;

            float absDecimal = Math.abs(decimalValue);

            d = (int) (Math.round(absDecimal + 0.5) - 1);
            int rest = Math.round(100 * 60.0f * (absDecimal - d));
            if (rest > 0) {
                m = rest / 100;
                dc = (rest - m * 100);

                // clean not used values
                if (m == 0) {
                    m = null;
                }
                if (dc == 0) {
                    dc = null;
                }
            }
            if (d == 0) {
                d = null;
            }
        }

        degree = d;
        minute = m;
        decimal = dc;
        sign = si;
    }

    public Float toDecimal() {
        if (isNull()) {
            return null;
        }
        Integer d = getNotNullDegree();
        Integer m = getNotNullMinute();
        Integer dc = getNotNullDecimal();
        Float result = Float.valueOf(d);
        result += (m + (dc / 100f)) / 60.0f;
        if (sign) {
            result *= -1;
        }
        return result;
    }

    public Integer getSignedDegree() {
        Integer result = null;
        if (!isDegreeNull()) {
            result = degree;
            if (isSign()) {
                result *= -1;
            }
        }
        return result;
    }

    public int getNotNullDegree() {
        return isDegreeNull() ? 0 : degree;
    }

    public int getNotNullMinute() {
        return isMinuteNull() ? 0 : minute;
    }

    public int getNotNullDecimal() {
        return isDecimalNull() ? 0 : decimal;
    }

    public boolean isLatitudeDegreeValid() {
        boolean result = isDegreeValid(false);
        return result;
    }

    public boolean isLongitudeDegreeValid() {
        boolean result = isDegreeValid(true);
        return result;
    }

    public boolean isMinuteValid() {
        boolean result = true;
        if (!isMinuteNull()) {
            if (minute == 60) {

                // can not have decimal
                result = isDecimalNull();
            } else {
                result = 0 <= minute && minute < 60;
            }
        }
        return result;
    }

    public boolean isDecimalValid() {
        boolean result = isDecimalNull() || (0 <= decimal && decimal < 100);
        return result;
    }

    @Override
    public String toString() {
        return "DmdCoordinateComponent{" +
               "sign=" + sign +
               ", degree=" + degree +
               ", minute=" + minute +
               ", decimal=" + decimal +
               '}';
    }

    protected boolean isDegreeValid(boolean longitude) {
        boolean result = true;
        if (!isDegreeNull()) {
            int bound = longitude ? 180 : 90;
            if (bound == degree) {

                // can not have minute nor decimal
                result = isMinuteNull() && isDecimalNull();
            } else {
                result = 0 <= degree && degree < bound;
            }
        }
        return result;
    }

}

==> tutti-ui-swing-2.8.1-sources.jar: fr/ifremer/tutti/ui/swing/spatial/DmdCoordinate.java <==
package fr.ifremer.tutti.ui.swing.spatial;

/*
 * #%L
 * Tutti :: UI
 * $Id: DmdCoordinate.java 1316 2013-10-25 16:43:35Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-2.8.1/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/spatial/DmdCoordinate.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import org.jdesktop.beans.AbstractSerializableBean;

/**
 * Geo coordinate in degree decimal, minute format.
 * <p/>
 * Created on 10/23/13.
 *
 * @author Tony Chemit <chemit@codelutin.com>
 * @since 2.8
 */
public class DmdCoordinate extends AbstractSerializableBean {

    private static final long serialVersionUID = 1L;


    public static final String PROPERTY_SIGN = "sign";

    public static final String PROPERTY_DEGREE = "degree";

    public static final String PROPERTY_MINUTE = "minute";

    public static final String PROPERTY_DECIMAL = "decimal";

    protected boolean sign;

    protected Integer degree;

    protected Integer minute;

    protected Integer decimal;

    public static DmdCoordinate empty() {
        DmdCoordinate r = new DmdCoordinate();
        return r;
    }

    /**
     * Methode statique de fabrique de position a partir d'une valeur du format
     * decimal.
     * <p/>
     * Note : Si la valeur (au format decimal) vaut <code>null</code>, alors on
     * reinitialise les composants de la position a <code>null</code> et la
     * methode {@link #isNull()} vaudra alors {@code true}.
     *
     * @param decimal la valeur au format decimal
     * @return une nouvelle instance de position convertie
     */
    public static DmdCoordinate valueOf(Float decimal) {
        DmdCoordinate r = new DmdCoordinate();
        r.fromDecimal(decimal);
        return r;
    }

    /**
     * Methode statique de fabrique de position a partir d'une valeur du format
     * degre décimale minute.
     *
     * @param d  la valeur des degres
     * @param m  la valeur des minutes
     * @param dc la valeur des décimales de minutes
     * @return une nouvelle instance de position convertie
     */
    public static DmdCoordinate valueOf(boolean sign, int d, int m, int dc) {
        DmdCoordinate r = new DmdCoordinate();
        r.setSign(sign);
        r.setDegree(d);
        r.setMinute(m);
        r.setDecimal(dc);
        return r;
    }

    public boolean isSign() {
        return sign;
    }

    public Integer getDegree() {
        return degree;
    }

    public Integer getMinute() {
        return minute;
    }

    public Integer getDecimal() {
        return decimal;
    }

    public void setSign(boolean sign) {
        Object oldValue = isSign();
        this.sign = sign;
        firePropertyChange(PROPERTY_SIGN, oldValue, sign);
    }

    public void setDegree(Integer degree) {
        Object oldValue = getDegree();
        this.degree = degree;
        firePropertyChange(PROPERTY_DEGREE, oldValue, degree);
    }

    public void setMinute(Integer minute) {
        Object oldValue = getMinute();
        this.minute = minute;
        firePropertyChange(PROPERTY_MINUTE, oldValue, minute);
    }

    public void setDecimal(Integer decimal) {
        Object oldValue = getDecimal();
        this.decimal = decimal;
        firePropertyChange(PROPERTY_DECIMAL, oldValue, decimal);
    }

    /**
     * @return {@code true} si aucune composante n'est renseignée,
     * {@code false} autrement.
     */
    public boolean isNull() {
        return degree == null && minute == null && decimal == null;
    }

    public boolean isDegreeNull() {
        return degree == null || degree == 0;
    }

    public boolean isMinuteNull() {
        return minute == null || minute == 0;
    }

    public boolean isDecimalNull() {
        return decimal == null || decimal == 0;
    }

    /**
     * Mets a jour les composants de la position a partir d'une valeur decimal.
     * <p/>
     * Note : Si la valeur (au format decimal) vaut <code>null</code>, alors on
     * reinitialise les composants de la position a <code>null</code> et la
     * methode {@link #isNull()} vaudra alors {@code true}.
     *
     * @param decimalValue la valeur decimale a convertir (qui peut etre nulle).
     */
    public void fromDecimal(Float decimalValue) {
        Integer d = null;
        Integer m = null;
        Integer dc = null;
        boolean si = false;
        if (decimalValue != null) {
            si = decimalValue < 0;

            float absDecimal = Math.abs(decimalValue);

            d = (int) (Math.round(absDecimal + 0.5) - 1);
            int rest = Math.round(100 * 60.0f * (absDecimal - d));
            if (rest > 0) {
                m = rest / 100;
                dc = (rest - m * 100);

                // clean not used values
                if (m == 0) {
                    m = null;
                }
                if (dc == 0) {
                    dc = null;
                }
            }
            if (d == 0) {
                d = null;
            }
        }

        degree = d;
        minute = m;
        decimal = dc;
        sign = si;
    }

    public Float toDecimal() {
        if (isNull()) {
            return null;
        }
        Integer d = getNotNullDegree();
        Integer m = getNotNullMinute();
        Integer dc = getNotNullDecimal();
        Float result = Float.valueOf(d);
        result += (m + (dc / 100f)) / 60.0f;
        if (sign) {
            result *= -1;
        }
        return result;
    }

    public Integer getSignedDegree() {
        Integer result = null;
        if (!isDegreeNull()) {
            result = degree;
            if (isSign()) {
                result *= -1;
            }
        }
        return result;
    }

    public int getNotNullDegree() {
        return isDegreeNull() ? 0 : degree;
    }

    public int getNotNullMinute() {
        return isMinuteNull() ? 0 : minute;
    }

    public int getNotNullDecimal() {
        return isDecimalNull() ? 0 : decimal;
    }

    public boolean isLatitudeDegreeValid() {
        boolean result = isDegreeValid(false);
        return result;
    }

    public boolean isLongitudeDegreeValid() {
        boolean result = isDegreeValid(true);
        return result;
    }

    public boolean isMinuteValid() {
        boolean result = true;
        if (!isMinuteNull()) {
            if (minute == 60) {

                // can not have decimal
                result = isDecimalNull();
            } else {
                result = 0 <= minute && minute < 60;
            }
        }
        return result;
    }

    public boolean isDecimalValid() {
        boolean result = isDecimalNull() || (0 <= decimal && decimal < 100);
        return result;
    }

    @Override
    public String toString() {
        return "DmdCoordinateComponent{" +
               "sign=" + sign +
               ", degree=" + degree +
               ", minute=" + minute +
               ", decimal=" + decimal +
               '}';
    }

    protected boolean isDegreeValid(boolean longitude) {
        boolean result = true;
        if (!isDegreeNull()) {
            int bound = longitude ? 180 : 90;
            if (bound == degree) {

                // can not have minute nor decimal
                result = isMinuteNull() && isDecimalNull();
            } else {
                result = 0 <= degree && degree < bound;
            }
        }
        return result;
    }

}
